﻿///******************
///* KillCounter mod for GTA 5
///* by spearminty
///***********************

using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Threading;

using System.Diagnostics;

using GTA;

namespace Killcounter
{
    public class KillCounter : Script
    {
        public KillCounter()
        {
            Tick += OnTick;
        }

        //list of all dead peds
        private List<int> PedList = new List<int>();
        //list of all dead vehicles
        private List<int> VehList = new List<int>();

        private int PedTally = 0;
        private int VehTally = 0;

        //fired when a kill occurs to flash the ui
        private bool PedZap = false;
        private bool VehZap = false;

        bool Killstreak = false;

        Ped Player;

        Stopwatch mTimer = new Stopwatch();

        //number of seconds before killstreak resets
        private int Cooldown = 15;
        //radius of area to check and qualify kills for the counter
        float Radius = 2000;

        /// <summary>
        /// on each game tick, perform this main loop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnTick(object sender, EventArgs e)
        {
            Player = Game.Player.Character;
            //every tick update UI if a killstreak is live
            if (Killstreak)
            {
                UpdateUI();
            }
            UpdateDeadPeds();
            UpdateDeadVehicles();
            CheckStreak();
        }

        /// <summary>
        /// check for and update dead pedestrians
        /// </summary>
        void UpdateDeadPeds()
        {
            Ped[] peds = World.GetNearbyPeds(Player, Radius);
            //for each entity in radius
            for (int i = 0; i < peds.Length; i++)
            {
                //check if its dead and damaged by us but not added previously
                if (peds[i].Exists() && peds[i].HasBeenDamagedBy(Player) && peds[i].IsDead)
                {
                    //check if we have added this dead ped yet
                    if (!PedList.Contains(peds[i].GetHashCode()))
                    {
                        PedTally = PedTally + 1;
                        PedList.Add(peds[i].GetHashCode());
                        ResetCooldown();
                        PedZap = true;
                    }
                }
            }
        }

        /// <summary>
        /// check for and update dead vehicles
        /// </summary>
        void UpdateDeadVehicles()
        {            
            Vehicle[] veh = World.GetNearbyVehicles(Player, Radius);
            //for each entity in radius
            for (int i = 0; i < veh.Length; i++)
            {
                //check if its dead and damaged by us but not added previously
                if (veh[i].Exists() && veh[i].HasBeenDamagedBy(Player) && !veh[i].IsDriveable)
                {
                    //check if we have added this dead veh yet
                    if (!VehList.Contains(veh[i].GetHashCode()))
                    {                        
                        VehTally = VehTally + 1;
                        VehList.Add(veh[i].GetHashCode());
                        ResetCooldown();
                        VehZap = true;
                    }
                }
            }
        }

        //check if we still qualify as a killstreak
        void CheckStreak()
        {
            //conditions for killstreak to go away:
            //time elapsed without kill  
            //player is dead
            if (mTimer.ElapsedMilliseconds > (Cooldown * 1000) || Player.IsDead)
            {
                Killstreak = false;
                VehTally = 0;
                PedTally = 0;
                mTimer.Stop();
                mTimer.Reset();
            }
        }

        //a kill has occurred, reset the timer
        void ResetCooldown()
        {            
            //reset the timer if its running, to continue the streak
            if (mTimer.IsRunning)
            {
                mTimer.Stop();
                mTimer.Reset();
                mTimer.Start();
            }
            else
            {
                //if it is the first kill, start timer
                mTimer.Start();
            }
            Killstreak = true;
        }

        //draw the UI
        private void UpdateUI()
        {
            //dimensions of eligible UI space to draw in
            int y = UI.HEIGHT;
            int x = UI.WIDTH;
            //adjusted x and y coords for where to draw the menus (for varying resolutions)
            int render_x = (int)Math.Round(x * 0.025);
            int render_y = (int)Math.Round(y * .25);
            //gap between text
            int gap = (int)(y * .035);            

            //pedestrian UI text:
            var pedColor = Color.Ivory;
            if (PedZap)
            {
                pedColor = Color.Red;
                PedZap = false;
            }
            string captionPed = "Pedestrians Killed: " + PedTally.ToString();

            var textPed = new UIText(captionPed, new Point(render_x, render_y + gap), .4f, pedColor, 7, false);
            var textPedBorder = new UIText(captionPed, new Point(render_x - 1, render_y + gap + 1), .41f, Color.Black, 7, false);

            //vehicles UI text:
            var vehColor = Color.Ivory;
            if (VehZap)
            {
                vehColor = Color.Red;
                VehZap = false;
            }
            string captionVeh = "Vehicles Killed: " + VehTally.ToString();

            var textVeh = new UIText(captionVeh, new Point(render_x, render_y + gap + 20), .4f, vehColor, 7, false);
            var textVehBorder = new UIText(captionVeh, new Point(render_x - 1, render_y + gap + 20 + 1), .41f, Color.Black, 7, false);

            //Killstreak + countdown text:
            //remaining time on killstreak
            decimal remaining = Math.Round((decimal)Cooldown - (decimal)(mTimer.ElapsedMilliseconds / 1000m), 2);
            
            //if less than 0, dont ever display a negative number
            if (remaining < 0)
            {
                remaining = 0;
            }

            string captionTimer = "Killstreak: " + remaining.ToString();

            var textTimer = new UIText(captionTimer, new Point(render_x, render_y), .5f, Color.WhiteSmoke, 7 ,false);
            var textTimerBorder = new UIText(captionTimer, new Point(render_x-1, render_y+1), .51f, Color.Black, 7 ,false);
           
            textTimerBorder.Draw();
            textTimer.Draw();

            textPedBorder.Draw();
            textPed.Draw();

            textVehBorder.Draw();
            textVeh.Draw();
        }
    }
}
